@@ -2,13 +2,15 @@ package bootstrap
 
 import (
 	"github.com/gophercloud/utils/openstack/clientconfig"
+
+	appsv1 "k8s.io/api/apps/v1"
 )
 
 type KuryrBootstrapResult struct {
 	ServiceSubnet            string
 	PodSubnetpool            string
 	WorkerNodesRouter        string
-	WorkerNodesSubnet        string
+	NodesNetworkMTU          int
 	PodSecurityGroups        []string
 	ExternalNetwork          string
 	ClusterID                string
@@ -24,7 +26,10 @@ type KuryrBootstrapResult struct {
 }
 
 type OVNBootstrapResult struct {
-	MasterIPs []string
+	MasterIPs               []string
+	ClusterInitiator        string
+	ExistingMasterDaemonset *appsv1.DaemonSet
+	ExistingNodeDaemonset   *appsv1.DaemonSet
 }
 
 type BootstrapResult struct {
