@@ -3,6 +3,8 @@ package network
 import (
 	"log"
 	"net"
+	"os"
+	"path/filepath"
 	"reflect"
 	"strings"
 
@@ -10,6 +12,7 @@ import (
 
 	operv1 "github.com/openshift/api/operator/v1"
 	"github.com/openshift/cluster-network-operator/pkg/bootstrap"
+	"github.com/openshift/cluster-network-operator/pkg/render"
 
 	uns "k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 )
@@ -19,35 +22,42 @@ func Render(conf *operv1.NetworkSpec, bootstrapResult *bootstrap.BootstrapResult
 	objs := []*uns.Unstructured{}
 
 	// render Multus
-	o, err := RenderMultus(conf, manifestDir)
+	o, err := renderMultus(conf, manifestDir)
 	if err != nil {
 		return nil, err
 	}
 	objs = append(objs, o...)
 
 	// render MultusAdmissionController
-	o, err = RenderMultusAdmissionController(conf, manifestDir)
+	o, err = renderMultusAdmissionController(conf, manifestDir)
 	if err != nil {
 		return nil, err
 	}
 	objs = append(objs, o...)
 
 	// render default network
-	o, err = RenderDefaultNetwork(conf, bootstrapResult, manifestDir)
+	o, err = renderDefaultNetwork(conf, bootstrapResult, manifestDir)
 	if err != nil {
 		return nil, err
 	}
 	objs = append(objs, o...)
 
 	// render kube-proxy
-	o, err = RenderStandaloneKubeProxy(conf, manifestDir)
+	o, err = renderStandaloneKubeProxy(conf, manifestDir)
 	if err != nil {
 		return nil, err
 	}
 	objs = append(objs, o...)
 
 	// render additional networks
-	o, err = RenderAdditionalNetworks(conf, manifestDir)
+	o, err = renderAdditionalNetworks(conf, manifestDir)
+	if err != nil {
+		return nil, err
+	}
+	objs = append(objs, o...)
+
+	// render network diagnostics
+	o, err = renderNetworkDiagnostics(conf, manifestDir)
 	if err != nil {
 		return nil, err
 	}
@@ -57,9 +67,9 @@ func Render(conf *operv1.NetworkSpec, bootstrapResult *bootstrap.BootstrapResult
 	return objs, nil
 }
 
-// CanonicalizeIPAMConfig converts configuration to a canonical form.
-// Currently we only care about case.
-func CanonicalizeIPAMConfig(conf *operv1.IPAMConfig) {
+// deprecatedCanonicalizeIPAMConfig converts configuration to a canonical form
+// for backward compatibility.
+func deprecatedCanonicalizeIPAMConfig(conf *operv1.IPAMConfig) {
 	switch strings.ToLower(string(conf.Type)) {
 	case strings.ToLower(string(operv1.IPAMTypeDHCP)):
 		conf.Type = operv1.IPAMTypeDHCP
@@ -68,9 +78,9 @@ func CanonicalizeIPAMConfig(conf *operv1.IPAMConfig) {
 	}
 }
 
-// CanonicalizeSimpleMacvlanConfig converts configuration to a canonical form.
-// Currently we only care about case.
-func CanonicalizeSimpleMacvlanConfig(conf *operv1.SimpleMacvlanConfig) {
+// deprecatedCanonicalizeSimpleMacvlanConfig converts configuration to a canonical form
+// for backward compatibility.
+func deprecatedCanonicalizeSimpleMacvlanConfig(conf *operv1.SimpleMacvlanConfig) {
 	switch strings.ToLower(string(conf.Mode)) {
 	case strings.ToLower(string(operv1.MacvlanModeBridge)):
 		conf.Mode = operv1.MacvlanModeBridge
@@ -83,13 +93,24 @@ func CanonicalizeSimpleMacvlanConfig(conf *operv1.SimpleMacvlanConfig) {
 	}
 
 	if conf.IPAMConfig != nil {
-		CanonicalizeIPAMConfig(conf.IPAMConfig)
+		deprecatedCanonicalizeIPAMConfig(conf.IPAMConfig)
 	}
 }
 
-// Canonicalize converts configuration to a canonical form.
-// Currently we only care about case.
-func Canonicalize(conf *operv1.NetworkSpec) {
+// DeprecatedCanonicalize converts configuration to a canonical form for backward
+// compatibility.
+//
+//      *** DO NOT ADD ANY NEW CANONICALIZATION TO THIS FUNCTION! ***
+//
+// Altering the user-provided configuration from CNO causes problems when other components
+// need to look at the configuration before CNO starts. Users should just write the
+// configuration in the correct form to begin with.
+//
+// However, we cannot remove any of the existing canonicalizations because this might
+// break existing clusters.
+func DeprecatedCanonicalize(conf *operv1.NetworkSpec) {
+	orig := conf.DeepCopy()
+
 	switch strings.ToLower(string(conf.DefaultNetwork.Type)) {
 	case strings.ToLower(string(operv1.NetworkTypeOpenShiftSDN)):
 		conf.DefaultNetwork.Type = operv1.NetworkTypeOpenShiftSDN
@@ -119,9 +140,14 @@ func Canonicalize(conf *operv1.NetworkSpec) {
 		}
 
 		if an.Type == operv1.NetworkTypeSimpleMacvlan && an.SimpleMacvlanConfig != nil {
-			CanonicalizeSimpleMacvlanConfig(conf.AdditionalNetworks[idx].SimpleMacvlanConfig)
+			deprecatedCanonicalizeSimpleMacvlanConfig(conf.AdditionalNetworks[idx].SimpleMacvlanConfig)
 		}
 	}
+
+	if !reflect.DeepEqual(orig, conf) {
+		log.Printf("WARNING: One or more fields of Network.operator.openshift.io was incorrectly capitalized. Although this has been fixed now, it is possible that other components previously saw the incorrect value and interpreted it incorrectly.")
+		log.Printf("Original spec: %#v\nModified spec: %#v\n", orig, conf)
+	}
 }
 
 // Validate checks that the supplied configuration is reasonable.
@@ -129,10 +155,10 @@ func Canonicalize(conf *operv1.NetworkSpec) {
 func Validate(conf *operv1.NetworkSpec) error {
 	errs := []error{}
 
-	errs = append(errs, ValidateIPPools(conf)...)
-	errs = append(errs, ValidateDefaultNetwork(conf)...)
-	errs = append(errs, ValidateMultus(conf)...)
-	errs = append(errs, ValidateStandaloneKubeProxy(conf)...)
+	errs = append(errs, validateIPPools(conf)...)
+	errs = append(errs, validateDefaultNetwork(conf)...)
+	errs = append(errs, validateMultus(conf)...)
+	errs = append(errs, validateKubeProxy(conf)...)
 
 	if len(errs) > 0 {
 		return errors.Errorf("invalid configuration: %v", errs)
@@ -146,7 +172,7 @@ func Validate(conf *operv1.NetworkSpec) error {
 // Defaults are carried forward from previous if it is provided. This is so we
 // can change defaults as we move forward, but won't disrupt existing clusters.
 func FillDefaults(conf, previous *operv1.NetworkSpec) {
-	hostMTU, err := GetDefaultMTU()
+	hostMTU, err := getDefaultMTU()
 	if hostMTU == 0 {
 		hostMTU = 1500
 	}
@@ -167,8 +193,8 @@ func FillDefaults(conf, previous *operv1.NetworkSpec) {
 		conf.LogLevel = "Normal"
 	}
 
-	FillDefaultNetworkDefaults(conf, previous, hostMTU)
-	FillKubeProxyDefaults(conf, previous)
+	fillDefaultNetworkDefaults(conf, previous, hostMTU)
+	fillKubeProxyDefaults(conf, previous)
 }
 
 // IsChangeSafe checks to see if the change between prev and next are allowed
@@ -198,7 +224,7 @@ func IsChangeSafe(prev, next *operv1.NetworkSpec) error {
 	}
 
 	// Check the default network
-	errs = append(errs, IsDefaultNetworkChangeSafe(prev, next)...)
+	errs = append(errs, isDefaultNetworkChangeSafe(prev, next)...)
 
 	// Changing AdditionalNetworks is supported
 
@@ -207,7 +233,7 @@ func IsChangeSafe(prev, next *operv1.NetworkSpec) error {
 	}
 
 	// Check kube-proxy
-	errs = append(errs, IsKubeProxyChangeSafe(prev, next)...)
+	errs = append(errs, isKubeProxyChangeSafe(prev, next)...)
 
 	if len(errs) > 0 {
 		return errors.Errorf("invalid configuration: %v", errs)
@@ -215,9 +241,9 @@ func IsChangeSafe(prev, next *operv1.NetworkSpec) error {
 	return nil
 }
 
-// ValidateIPPools checks that all IP addresses are valid
+// validateIPPools checks that all IP addresses are valid
 // TODO: check for overlap
-func ValidateIPPools(conf *operv1.NetworkSpec) []error {
+func validateIPPools(conf *operv1.NetworkSpec) []error {
 	errs := []error{}
 	for idx, pool := range conf.ClusterNetwork {
 		_, _, err := net.ParseCIDR(pool.CIDR)
@@ -235,8 +261,8 @@ func ValidateIPPools(conf *operv1.NetworkSpec) []error {
 	return errs
 }
 
-// ValidateMultus validates the combination of DisableMultiNetwork and AddtionalNetworks
-func ValidateMultus(conf *operv1.NetworkSpec) []error {
+// validateMultus validates the combination of DisableMultiNetwork and AddtionalNetworks
+func validateMultus(conf *operv1.NetworkSpec) []error {
 	// DisableMultiNetwork defaults to false
 	deployMultus := true
 	if conf.DisableMultiNetwork != nil && *conf.DisableMultiNetwork {
@@ -251,9 +277,9 @@ func ValidateMultus(conf *operv1.NetworkSpec) []error {
 	return []error{}
 }
 
-// ValidateDefaultNetwork validates whichever network is specified
+// validateDefaultNetwork validates whichever network is specified
 // as the default network.
-func ValidateDefaultNetwork(conf *operv1.NetworkSpec) []error {
+func validateDefaultNetwork(conf *operv1.NetworkSpec) []error {
 	switch conf.DefaultNetwork.Type {
 	case operv1.NetworkTypeOpenShiftSDN:
 		return validateOpenShiftSDN(conf)
@@ -266,11 +292,11 @@ func ValidateDefaultNetwork(conf *operv1.NetworkSpec) []error {
 	}
 }
 
-// RenderDefaultNetwork generates the manifests corresponding to the requested
+// renderDefaultNetwork generates the manifests corresponding to the requested
 // default network
-func RenderDefaultNetwork(conf *operv1.NetworkSpec, bootstrapResult *bootstrap.BootstrapResult, manifestDir string) ([]*uns.Unstructured, error) {
+func renderDefaultNetwork(conf *operv1.NetworkSpec, bootstrapResult *bootstrap.BootstrapResult, manifestDir string) ([]*uns.Unstructured, error) {
 	dn := conf.DefaultNetwork
-	if errs := ValidateDefaultNetwork(conf); len(errs) > 0 {
+	if errs := validateDefaultNetwork(conf); len(errs) > 0 {
 		return nil, errors.Errorf("invalid Default Network configuration: %v", errs)
 	}
 
@@ -287,8 +313,7 @@ func RenderDefaultNetwork(conf *operv1.NetworkSpec, bootstrapResult *bootstrap.B
 	}
 }
 
-// FillDefaultNetworkDefaults
-func FillDefaultNetworkDefaults(conf, previous *operv1.NetworkSpec, hostMTU int) {
+func fillDefaultNetworkDefaults(conf, previous *operv1.NetworkSpec, hostMTU int) {
 	switch conf.DefaultNetwork.Type {
 	case operv1.NetworkTypeOpenShiftSDN:
 		fillOpenShiftSDNDefaults(conf, previous, hostMTU)
@@ -300,7 +325,7 @@ func FillDefaultNetworkDefaults(conf, previous *operv1.NetworkSpec, hostMTU int)
 	}
 }
 
-func IsDefaultNetworkChangeSafe(prev, next *operv1.NetworkSpec) []error {
+func isDefaultNetworkChangeSafe(prev, next *operv1.NetworkSpec) []error {
 	if prev.DefaultNetwork.Type != next.DefaultNetwork.Type {
 		return []error{errors.Errorf("cannot change default network type")}
 	}
@@ -318,7 +343,7 @@ func IsDefaultNetworkChangeSafe(prev, next *operv1.NetworkSpec) []error {
 }
 
 // ValidateAdditionalNetworks validates additional networks configs
-func ValidateAdditionalNetworks(conf *operv1.NetworkSpec) []error {
+func validateAdditionalNetworks(conf *operv1.NetworkSpec) []error {
 	out := []error{}
 	ans := conf.AdditionalNetworks
 	for _, an := range ans {
@@ -338,15 +363,15 @@ func ValidateAdditionalNetworks(conf *operv1.NetworkSpec) []error {
 	return out
 }
 
-// RenderAdditionalNetworks generates the manifests of the requested additional networks
-func RenderAdditionalNetworks(conf *operv1.NetworkSpec, manifestDir string) ([]*uns.Unstructured, error) {
+// renderAdditionalNetworks generates the manifests of the requested additional networks
+func renderAdditionalNetworks(conf *operv1.NetworkSpec, manifestDir string) ([]*uns.Unstructured, error) {
 	var err error
 	ans := conf.AdditionalNetworks
 	out := []*uns.Unstructured{}
 	objs := []*uns.Unstructured{}
 
 	// validate additional network configuration
-	if errs := ValidateAdditionalNetworks(conf); len(errs) > 0 {
+	if errs := validateAdditionalNetworks(conf); len(errs) > 0 {
 		return nil, errors.Errorf("invalid Additional Network Configuration: %v", errs)
 	}
 
@@ -377,8 +402,8 @@ func RenderAdditionalNetworks(conf *operv1.NetworkSpec, manifestDir string) ([]*
 	return out, nil
 }
 
-// RenderMultusAdmissionController generates the manifests of Multus Admission Controller
-func RenderMultusAdmissionController(conf *operv1.NetworkSpec, manifestDir string) ([]*uns.Unstructured, error) {
+// renderMultusAdmissionController generates the manifests of Multus Admission Controller
+func renderMultusAdmissionController(conf *operv1.NetworkSpec, manifestDir string) ([]*uns.Unstructured, error) {
 	if *conf.DisableMultiNetwork {
 		return nil, nil
 	}
@@ -394,3 +419,22 @@ func RenderMultusAdmissionController(conf *operv1.NetworkSpec, manifestDir strin
 	out = append(out, objs...)
 	return out, nil
 }
+
+// renderNetworkDiagnostics renders the connectivity checks
+func renderNetworkDiagnostics(conf *operv1.NetworkSpec, manifestDir string) ([]*uns.Unstructured, error) {
+	if conf.DisableNetworkDiagnostics {
+		return nil, nil
+	}
+
+	data := render.MakeRenderData()
+	data.Data["ReleaseVersion"] = os.Getenv("RELEASE_VERSION")
+	data.Data["NetworkCheckSourceImage"] = os.Getenv("NETWORK_CHECK_SOURCE_IMAGE")
+	data.Data["NetworkCheckTargetImage"] = os.Getenv("NETWORK_CHECK_TARGET_IMAGE")
+
+	manifests, err := render.RenderDir(filepath.Join(manifestDir, "network-diagnostics"), &data)
+	if err != nil {
+		return nil, errors.Wrap(err, "failed to render network-diagnostics manifests")
+	}
+
+	return manifests, nil
+}
