@@ -9,10 +9,10 @@ import (
 
 	"github.com/ghodss/yaml"
 	operv1 "github.com/openshift/api/operator/v1"
-
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	utilerrors "k8s.io/apimachinery/pkg/util/errors"
 	utilnet "k8s.io/apimachinery/pkg/util/net"
+	cliflag "k8s.io/component-base/cli/flag"
 	kubeproxyconfig "k8s.io/kube-proxy/config/v1alpha1"
 )
 
@@ -80,6 +80,7 @@ func GenerateKubeProxyConfiguration(args map[string]operv1.ProxyArgumentList) (s
 	kpc.ConfigSyncPeriod.Duration = ka.getDuration("config-sync-period")
 
 	kpc.NodePortAddresses = ka.getCIDRList("node-port-addresses")
+	kpc.FeatureGates = ka.getFeatureGates("feature-gates")
 
 	if err := ka.getError(); err != nil {
 		return "", err
@@ -125,31 +126,32 @@ func (ka *kpcArgs) get(argName string) string {
 // getAddressAndPort returns a combined IP address and port
 func (ka *kpcArgs) getAddressAndPort(addressKey, portKey, defaultPort string) string {
 	address := ka.get(addressKey)
+	port := ka.get(portKey)
+
+	if address == "" && port == "" {
+		return ""
+	}
+
 	if address != "" {
 		if net.ParseIP(address) == nil {
 			ka.errs = append(ka.errs, fmt.Errorf("invalid %s %q (not an IP address)", addressKey, address))
 			return ""
 		}
+	} else {
+		// default to 0.0.0.0
+		address = "0.0.0.0"
 	}
-	port := ka.get(portKey)
+
 	if port != "" {
-		if _, err := strconv.ParseInt(port, 10, 16); err != nil {
+		if _, err := strconv.ParseUint(port, 10, 16); err != nil {
 			ka.errs = append(ka.errs, fmt.Errorf("invalid %s %q (%v)", portKey, port, err))
 			return ""
 		}
+	} else {
+		port = defaultPort
 	}
 
-	if address != "" {
-		if port == "" {
-			return address + ":" + defaultPort
-		} else {
-			return address + ":" + port
-		}
-	} else if port != "" {
-		return "0.0.0.0:" + port
-	} else {
-		return ""
-	}
+	return net.JoinHostPort(address, port)
 }
 
 // getString returns an uninterpreted string
@@ -165,6 +167,7 @@ func (ka *kpcArgs) getAddress(key string) string {
 	}
 	if net.ParseIP(value) == nil {
 		ka.errs = append(ka.errs, fmt.Errorf("invalid %s %q (not an IP address)", key, value))
+		return ""
 	}
 	return value
 }
@@ -253,3 +256,26 @@ func (ka *kpcArgs) getPortRange(key string) string {
 	}
 	return value
 }
+
+// getFeatureGates parses feature-gates and returns a map[string]bool
+func (ka *kpcArgs) getFeatureGates(key string) map[string]bool {
+	val := ka.args[key]
+	if len(val) == 0 {
+		return nil
+	}
+	delete(ka.args, key)
+	fgMap := make(map[string]bool)
+	featureGates := cliflag.NewMapStringBool(&fgMap)
+	for _, v := range val {
+		err := featureGates.Set(v)
+		if err != nil {
+			ka.errs = append(ka.errs, fmt.Errorf("invalid %q (%v)", v, err))
+			continue
+		}
+	}
+	if len(fgMap) == 0 {
+		return nil
+	}
+	return fgMap
+
+}
